/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#ifndef __CParticleFilter_H__
#define __CParticleFilter_H__
//-----------------------------------------------------------
#include <vector>
using namespace std;

#include "Particle.h"

//-----------------------------------------------------------
/**
   \struct 
   \brief データをやり取りするためにSourceの代わりとして作った構造体
*/
typedef struct SOURCE
{
	float x[3];
	// Changed by Morio 3D拡張
	float azimuth;
	float elevation;
	//float angle;
	float power; //とりあえずpowとして入ってくることにしておく
	//int Frame_ID;
	int id;

	//TFIndex 2016.9.26
    int tfindex[3];


	//float output_theta;
} SOURCE;
//-----------------------------------------------------------

class CParticleFilter {
private:
	//FILE  *fp_output;
	robot_observation_set r_set;///<robの観測値
	observation_set ob_set;		///<roomの観測値
	particle_group_set pg_set; ///<粒子グループのセット

/** 
 * \brief 入力データ(観測値)をセットする
 * \param flag ノードのフラグ
 */
	void SetData(const int flag);

	///パラメータ構造体
	PARAM s_param;


public:
	CParticleFilter();	///<コンストラクタ　パラメータ処理など
	~CParticleFilter();	///<デストラクタ

	FILE *fp_file;

	vector<SOURCE> input;  ///<データ入力用のベクタ(room)
	vector<SOURCE> input_r;  ///<データ入力用のベクタ(rob)

	vector<SOURCE> output;	///<データ出力用のベクタ

	SOURCE src_rob;	
	SOURCE src_room;
	SOURCE s_out;	///<アウトプット

	int i_ob_num_IRMA;	///<観測値の数(室内)
	int i_ob_num_REMA;	///<観測値の数(ロボット)

	int i_FrameCount;	///<処理回数（≒フレーム番号）

	/**
	\brief \~Japanese パラメータの初期化
	*/
	void initial_param(const int flag);

	/**
	\brief \~Japanese パラメータのファイル出力
	*/
	void print_param();

	/**
	\brief \~Japanese パラメータの設定
	*/
	void set_param(	const int param_particle_num, 
					const int param_max_num_of_source,
					const double param_th_another_source,
					const double param_th_ignore_source,
					const int param_th_remove_source,
					const double param_th_output_range, 
					const double param_likelihood_sigma,
					const double param_likelihood_sigma_robot,
					const double param_likelihood_robot_weight,
					const double param_th_random_walk,
					const double param_state_update_sigma,
					const double param_sum_weight,		
					const double param_rob_pow,
					const double param_room_length, 
					const double param_room_width, 
					const double param_room_pow,
					const bool param_history);

	/**
	\brief \~Japanese ParticleFilterの実行
	*/
	void ExecuteParticleFilter(const int flag);	

};
#endif
