/*
 * Copyright (c) 2004 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#ifndef	PF_EST_REV_h_defined
#define	PF_EST_REV_h_defined
// ---------------------------------------------------------------------
#include "MICARYHA_struct.h"
#include "MICARY_PF_Struct.h"

// 2007/07/13 added by S.Suzuki calling C from C++
#ifdef __cplusplus
extern "C" {
#endif

// ---------------------------------------------------------------------

/**
 \brief \~Japanese 残響推定(1:正常終了, 0:エラー)
 \param fsInputPSpec \~English Input power spectrum (N, M)
 \~Japanese 入力パワー
 \param sPrmRev \~English Parameters for Estimation reverberation noise calc.
 \~Japanese 残響推定計算のためのパラメータ
 \param psSavedRev \~English Estimated reverberation noise power spectrum of previous frame (N ,M)
 \~Japanese 前フレームの残響パワー
 \param pfsEstRevPSpec \~English Total estimated reverberation power spectrum (N, M)
 \~Japanese 推定した残響成分パワー
*/
int PF_EstRev(
		MICARY_FloatSpecArray fsInputPSpec,
		MICARY_PrmRevNoise sPrmRev,
		MICARY_SavedEstRev *psSavedRev,
		MICARY_FloatSpecArray *pfsEstRevPSpec);

#ifdef __cplusplus
}
#endif

#endif	// PF_EST_REV_h_defined
